require 'random-accessible'
require 'psych'

# Read and write YAML files in a directory like an Array.
class YAMLDir

  include RandomAccessible

  def initialize(dirname)
    @dirname = dirname
    @size = 0

    unless Dir.exist?(dirname)
      Dir.mkdir(dirname)
    end

    Dir.foreach(dirname) do |name|
      path = File.join(dirname, name)
      #if name =~ /^(\d+).[(yaml)|(yml)]/
      if name =~ /^(\d+).yaml/
        unless FileTest.directory?(path)
          index = Regexp.last_match[1].to_i
          @size = [@size, index].max
        end
      end
    end
  end

  def read_access(pos)
    filename = File.join(@dirname, "#{pos}.yaml")
    unless FileTest.exist?(filename)
      return nil
    end

    return Psych.load_file(filename)
  end

  def replace_access(pos, val)
    filename = File.join(@dirname, "#{pos}.yaml")
    File.open(filename, "w") do |file|
      Psych.dump(val, file)
    end

    @size = [@size, pos + 1].max
  end

  def shrink(n)
    n.times do
      pos = @size - 1
      filename = File.join(@dirname, "#{pos}.yaml")
      if FileTest.exist?(filename)
        File.delete(filename)
      end
      @size -= 1
    end
  end

  attr_reader :size

end

yd = YAMLDir.new ARGV[0]
10.times do |i|
  yd[i] = i
end
5.times do |i|
  yd.delete_at(yd.size - 1)
end
yd.each_with_index do |i|
  puts "#{i}: #{yd[i]}"
end

